import re


ERROR_RX = re.compile("# ((GQL[0-9]+ ?)+)(: (.*))?$")


def extract_expected_errors(data):
    lines = data.splitlines()
    expected_codes = []
    expected_messages = []
    for line in lines:
        match = ERROR_RX.search(line)
        if match:
            codes = match.group(1).split()
            message = match.group(4)
            expected_codes.extend(codes)
            if message:
              expected_messages.append(message)
    return expected_codes, expected_messages



from __future__ import absolute_import, division, print_function

__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "gql-checker"
__summary__ = (
    "Flake8 and pylama plugin that checks gql GraphQL calls."
)
__uri__ = "https://github.com/graphql-python/gql-checker"

__version__ = "0.1"

__author__ = "Syrus Akbary"
__email__ = "me@syrusakbary.com"

__license__ = "MIT"
__copyright__ = "Copyright 2016 %s" % __author__



import ast
import re
import os

import pycodestyle
import pytest

from gql_checker.flake8_linter import Linter

from tests.utils import extract_expected_errors


def load_test_cases():
    base_path = os.path.dirname(__file__)
    test_case_path = os.path.join(base_path, "test_cases")
    test_case_files = os.listdir(test_case_path)

    test_cases = []

    for fname in test_case_files:
        if not fname.endswith(".py"):
            continue

        fullpath = os.path.join(test_case_path, fname)
        data = open(fullpath).read()
        tree = ast.parse(data, fullpath)
        codes, messages = extract_expected_errors(data)

        test_cases.append((tree, fullpath, codes, messages))

    return test_cases


@pytest.mark.parametrize(
    "tree, filename, expected_codes, expected_messages",
    load_test_cases()
)
def test_expected_error(tree, filename, expected_codes, expected_messages):
    argv = [
        "--gql-introspection-schema=./tests/introspection_schema.json"
    ]

    parser = pycodestyle.get_parser('', '')
    Linter.add_options(parser)
    options, args = parser.parse_args(argv)
    Linter.parse_options(options)

    checker = Linter(tree, filename)
    codes = []
    messages = []
    for lineno, col_offset, msg, instance in checker.run():
        code, message = msg.split(" ", 1)
        codes.append(code)
        messages.append(message)
    assert codes == expected_codes
    assert set(messages) >= set(expected_messages)



from __future__ import absolute_import

import gql_checker
from gql_checker import ImportOrderChecker


class Linter(ImportOrderChecker):
    name = "gql"
    version = gql_checker.__version__

    def __init__(self, tree, filename):
        super(Linter, self).__init__(filename, tree)

    @classmethod
    def add_options(cls, parser):
        # List of application import names. They go last.
        parser.add_option(
            "--gql-introspection-schema",
            metavar="FILE",
            help="Import names to consider as application specific"
        )
        parser.add_option(
            "--gql-typedef-schema",
            default='',
            action="store",
            type="string",
            help=("Style to follow. Available: "
                  "cryptography, google, smarkets, pep8")
        )
        parser.config_options.append("gql-introspection-schema")
        parser.config_options.append("gql-typedef-schema")

    @classmethod
    def parse_options(cls, options):
        optdict = {}

        optdict = dict(
            gql_introspection_schema=options.gql_introspection_schema,
            gql_typedef_schema=options.gql_typedef_schema,
        )

        cls.options = optdict

    def error(self, node, code, message):
        lineno, col_offset = node.lineno, node.col_offset
        return (lineno, col_offset, '{0} {1}'.format(code, message), Linter)

    def run(self):
        for error in self.check_gql():
            yield error



import ast
import os

import pytest

from gql_checker import pylama_linter

from tests.utils import extract_expected_errors


def load_test_cases():
    base_path = os.path.dirname(__file__)
    test_case_path = os.path.join(base_path, "test_cases")
    test_case_files = os.listdir(test_case_path)

    test_cases = []

    for fname in test_case_files:
        if not fname.endswith(".py"):
            continue

        fullpath = os.path.join(test_case_path, fname)
        data = open(fullpath).read()
        codes, messages = extract_expected_errors(data)
        test_cases.append((fullpath, codes, messages))

    return test_cases


@pytest.mark.parametrize(
    "filename, expected_codes, expected_messages",
    load_test_cases()
)
def test_expected_error(filename, expected_codes, expected_messages):
    checker = pylama_linter.Linter()
    assert checker.allow(filename)

    codes = []
    messages = []

    options = {
        "gql_introspection_schema": "./tests/introspection_schema.json"
    }

    for error in checker.run(filename, **options):
        codes.append(error['type'])
        messages.append(error['text'])

    assert codes == expected_codes
    assert set(messages) >= set(expected_messages)



import ast
import json

import pycodestyle

from gql_checker.__about__ import (
    __author__, __copyright__, __email__, __license__, __summary__, __title__,
    __uri__, __version__
)
from gql_checker.stdlib_list import STDLIB_NAMES
from graphql import Source, validate, parse, build_client_schema


__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

GQL_SYNTAX_ERROR = 'GQL100'
GQL_VALIDATION_ERROR = 'GQL101'

class ImportVisitor(ast.NodeVisitor):
    """
    This class visits all the gql calls.
    """

    def __init__(self, filename, options):
        self.filename = filename
        self.options = options or {}
        self.calls = []

    def visit_Call(self, node):  # noqa
        if node.func.id == 'gql':
            self.calls.append(node)

    def node_query(self, node):
        """
        Return the query for the gql call node
        """

        if isinstance(node, ast.Call):
            assert node.args
            arg = node.args[0]
            if not isinstance(arg, ast.Str):
                return
        else:
            raise TypeError(type(node))

        return arg.s


class ImportOrderChecker(object):
    visitor_class = ImportVisitor
    options = None

    def __init__(self, filename, tree):
        self.tree = tree
        self.filename = filename
        self.lines = None

    def load_file(self):
        if self.filename in ("stdin", "-", None):
            self.filename = "stdin"
            self.lines = pycodestyle.stdin_get_value().splitlines(True)
        else:
            self.lines = pycodestyle.readlines(self.filename)

        if not self.tree:
            self.tree = ast.parse("".join(self.lines))

    def get_schema(self):
        gql_introspection_schema = self.options.get('gql_introspection_schema')
        if gql_introspection_schema:
            try:
                with open(gql_introspection_schema) as data_file:
                    introspection_schema = json.load(data_file)
                    return build_client_schema(introspection_schema)
            except IOError as e:
                raise Exception("Cannot find the provided introspection schema. {}".format(str(e)))

        schema = self.options.get('schema')
        assert schema, 'Need to provide schema'

    def validation_errors(self, ast):
        return validate(self.get_schema(), ast)

    def error(self, node, code, message):
        raise NotImplemented()

    def check_gql(self):
        if not self.tree or not self.lines:
            self.load_file()

        visitor = self.visitor_class(self.filename, self.options)
        visitor.visit(self.tree)

        for node in visitor.calls:
            # Lines with the noqa flag are ignored entirely
            if pycodestyle.noqa(self.lines[node.lineno - 1]):
                continue

            query = visitor.node_query(node)
            if not query:
                continue

            try:
                source = Source(query, 'gql query')
                ast = parse(source)
            except Exception as e:
                message = str(e)
                yield self.error(node, GQL_SYNTAX_ERROR, message)
                continue

            validation_errors = self.validation_errors(ast)
            if validation_errors:
                for error in validation_errors:
                    message = str(error)
                    yield self.error(node, GQL_VALIDATION_ERROR, message)



from gql import gql


gql('''
    query NestedQueryWithFragment {
      hero {
        ...NameAndAppearances
        friends {
          ...NameAndAppearances
          friends {
            ...NameAndAppearances
          }
        }
      }
    }
    fragment NameAndAppearances on Character {
      name
      appearsIn
    }
''')

gql('''
    query HeroSpaceshipQuery {
      hero {
        favoriteSpaceship
      }
    }
''') # GQL101: Cannot query field "favoriteSpaceship" on type "Character".

gql('''
    query HeroNoFieldsQuery {
      hero
    }
''') # GQL101: Field "hero" of type "Character" must have a sub selection.


gql('''
    query HeroFieldsOnScalarQuery {
      hero {
        name {
          firstCharacterOfName
        }
      }
    }
''') # GQL101: Field "name" of type "String" must not have a sub selection.


gql('''
    query DroidFieldOnCharacter {
      hero {
        name
        primaryFunction
      }
    }
''') # GQL101: Cannot query field "primaryFunction" on type "Character". However, this field exists on "Droid". Perhaps you meant to use an inline fragment?

gql('''
    query DroidFieldInFragment {
      hero {
        name
        ...DroidFields
      }
    }
    fragment DroidFields on Droid {
      primaryFunction
    }
''')

gql('''
    query DroidFieldInFragment {
      hero {
        name
        ... on Droid {
          primaryFunction
        }
      }
    }
''')
